from CloudFlare import CloudFlare
from CloudFlare.exceptions import CloudFlareAPIError

from check_ip.config import ConfigError


class CloudFlareAPIClientError(Exception):
    """Raised when the client cannot fulfill a request."""


class CloudflareAPIClient:
    def __init__(self, email, api_key):
        self.client = CloudFlare(email=email, token=api_key)
        self._validate_auth()

    def _validate_auth(self):
        try:
            self.client.user.get()
        except CloudFlareAPIError as err:
            raise ConfigError(
                "Authentication information incorrect, please check your configuration file."
            ) from err

    def get_dns_record(self, zone_name, record_name):
        """Get a DNS record in a given zone."""
        all_dns_records = self._get_dns_records(zone_name)
        if record_name not in all_dns_records:
            raise CloudFlareAPIClientError(
                f"Could not find a DNS record with name '{record_name}'."
            )
        return all_dns_records[record_name]

    def update_dns_record(self, zone_name, record_name, new_content):
        """Update a DNS record in a given zone with the given name.

        Assumes an "A" record.
        """
        zone = self._get_zone(zone_name)
        record = self.get_dns_record(zone_name, record_name)

        return self.client.zones.dns_records.put(
            zone["id"],
            record["id"],
            data={"name": record_name, "type": "A", "content": new_content},
        )

    def _get_zone(self, zone_name):
        """Get a zone by name."""
        zones = self.client.zones.get(params={"name": zone_name})
        if not zones:
            raise CloudFlareAPIClientError(
                f"Could not find zone with name '{zone_name}'."
            )
        return zones[0]

    def _get_dns_records(self, zone_name):
        """Get DNS records in a given zone.

        Only get "A" records.
        """
        zone = self._get_zone(zone_name)

        all_dns_records = self.client.zones.dns_records.get(zone["id"])
        return {
            record["name"]: record
            for record in all_dns_records
            if record["type"] == "A"  # Only need to look at A records.
        }



import click
from requests import RequestException

from check_ip.api import get_public_ip
from check_ip.api.cloudflare import CloudflareAPIClient
from check_ip.api.cloudflare import CloudFlareAPIClientError
from check_ip.config import Config
from check_ip.config import ConfigError


@click.command()
@click.argument("config_file")
@click.option("-v", "--verbose", is_flag=True)
@click.pass_context
def main(ctx, config_file, verbose):
    """Check your public IP address and update DNS records on Cloudflare."""
    # Load config.
    try:
        config = Config.from_file(config_file)
        config.verbose = verbose
        cloudflare_api_client = CloudflareAPIClient(config.email, config.api_key)

        # Get the current public IP.
        public_ip = get_public_ip()

        # For each record name in the config, check the content and update if necessary.
        for record_name in config.record_names:
            record = cloudflare_api_client.get_dns_record(config.zone, record_name)

            if record["content"] == public_ip:
                if verbose:
                    click.echo(
                        f"Public IP matches {record_name} ({record['content']})",
                        err=True,
                    )
                continue
            click.echo(
                f"Public IP ({public_ip}) does not match {record_name} ({record['content']})",
                err=True,
            )

            # Update the record IP address.
            cloudflare_api_client.update_dns_record(config.zone, record_name, public_ip)
            click.echo(
                f"Updated record for {record['name']}: {record['content']} -> {public_ip}",
                err=True,
            )
    except (ConfigError, RequestException, CloudFlareAPIClientError) as err:
        ctx.fail(err)



from setuptools import find_packages
from setuptools import setup

with open("README.md") as f:
    README = f.read()

setup(
    name="check-ip",
    version="0.4.5",
    description="Check your public IP address and update DNS records on Cloudflare.",
    long_description=README,
    long_description_content_type="text/markdown",
    author="Samuel Searles-Bryant",
    author_email="devel@samueljsb.co.uk",
    license="MIT",
    url="https://github.com/samueljsb/check-ip/",
    packages=find_packages(),
    install_requires=["click", "cloudflare", "pyyaml", "requests"],
    entry_points={"console_scripts": ["check-ip=check_ip.cli:main"]},
)



from dataclasses import dataclass
from typing import List

import yaml


class ConfigError(Exception):
    """Raised when there is a problem with the configuration."""


@dataclass
class Config:
    email: str
    api_key: str
    zone: str
    records: List[str]
    verbose: bool = False

    @property
    def record_names(self):
        return [f"{record}.{self.zone}" for record in self.records]

    @classmethod
    def from_file(cls, filepath):
        try:
            with open(filepath, "r") as f:
                _config = yaml.safe_load(f)
        except FileNotFoundError as err:
            raise ConfigError(f"Config file not found: '{filepath}'") from err
        try:
            config = cls(**_config)
        except TypeError as err:
            raise ConfigError("Config file is missing values.") from err
        return config
